use strict;
use warnings;
use Test::More;

require_ok 'App::Prove::Plugin::ShareDirDist';
require_ok 'App::Yath::Plugin::ShareDirDist';
require_ok 'File::ShareDir::Dist';
require_ok 'File::ShareDir::Dist::Install';

done_testing;

